import sys
import tempfile
import re, os
import popen2
import pickle

indent_re = re.compile("^( *)[^ ]*.*$")

def write_report(test_output):
    abuf = tempfile.NamedTemporaryFile()
    new_name = abuf.name+"_1"
    open(new_name, 'w').writelines(test_output)
    return new_name


class TestClassFinder(object):
    def __init__(self, tabstop):
        self._clsname = None
        self.class_re  = re.compile("class  *(.*) *\(.")
        self.tabstop = float(tabstop)

    def satisfied(self):
        return self._clsname is not None

    def visit(self, line):
        if self.satisfied():
            return
        re_match = indent_re.match(line)
        indent_size = len(re_match.groups()[0])
        indent_level = indent_size / self.tabstop
        syntax_part = line[indent_size:]
        if indent_level == 0.0 and syntax_part.startswith("class Test"):
            self._clsname = self.class_re.match(syntax_part).groups()[0]

    def test_part(self):
        return self._clsname

class TestMethodFinder(object):
    def __init__(self, tabstop):
        self._methodname = None
        self.method_re = re.compile("def  *(.*) *\(.")
        self.tabstop = float(tabstop)

    def satisfied(self):
        return self._methodname is not None

    def visit(self, line):
        if self.satisfied():
            return
        re_match = indent_re.match(line)
        indent_size = len(re_match.groups()[0])
        indent_level = indent_size / self.tabstop
        syntax_part = line[indent_size:]
        if indent_level == 1.0 and syntax_part.startswith("def test"):
            # this is a test method
            self._methodname = self.method_re.match(syntax_part).groups()[0]

    def test_part(self):
        return self._methodname

class TestFunctionFinder(object):
    def __init__(self, tabstop):
        self._funcname = None
        self.method_re = re.compile("def  *(.*) *\(.")
        self.tabstop = float(tabstop)

    def satisfied(self):
        return self._funcname is not None

    def visit(self, line):
        if self.satisfied():
            return
        re_match = indent_re.match(line)
        indent_size = len(re_match.groups()[0])
        indent_level = indent_size / self.tabstop
        syntax_part = line[indent_size:]

        if indent_level == 0.0 and syntax_part.startswith("def test"):
            self._funcname = self.method_re.match(syntax_part).groups()[0]

    def test_part(self):
        return self._funcname

class TestThisFinder(object):
    '''
    This is a composite finder of teh Function and the class/method finders
    '''
    def __init__(self, tabstop):
        self._meth_find = TestMethodFinder(tabstop)
        self._cls_find = TestClassFinder(tabstop)
        self._func_find = TestFunctionFinder(tabstop)

    def satisfied(self):
        result =  (self._cls_find.satisfied() and self._meth_find.satisfied()) or \
                   self._cls_find.satisfied() or \
                   self._func_find.satisfied()
        return result

    def visit(self, line):
        self._meth_find.visit(line)
        self._cls_find.visit(line)
        self._func_find.visit(line)

    def test_part(self):
        if (self._cls_find.satisfied() and self._meth_find.satisfied()):
            return "%s.%s" % (self._cls_find.test_part(), self._meth_find.test_part())
        if self._cls_find.satisfied():
            return "%s" % self._cls_find.test_part()
        if self._func_find.satisfied():
            return self._func_find.test_part()

class TestBuilder(object):
    '''
    This class captures a bunch of state information from the active
    buffer and will invoke nosetests 
    '''
    def __init__(self, visitor, filename, line_num):
        self.testfunc_name = None
        self.testmethod_name = None
        self.testclass_name = None
        
        self.row = int(line_num)
        self.content = open(filename).readlines()

        self._visitor = visitor

    def iterate_content(self):
        for line in self.content[self.row-1::-1]:
            self._visitor.visit(line)
            if self._visitor.satisfied():
                return self._visitor.test_part()

def nosecmd():
    with_growl = ''
    with_django = ''
    try:
        import nosegrowl
        with_growl = '--with-growl'
    except:
        pass
    try:
        import nosedjango
        with_django = '--with-growl'
    except:
        pass

    return "nosetests %s %s -sv " % (with_growl, with_django)
    

def test_this():
    filename, tabstop, line_num = sys.argv[-3:]

    tb = TestBuilder(TestThisFinder(tabstop), filename, line_num)
    testname = tb.iterate_content()

    if not testname:
        return test_module()

    cmd = "%s %s:%s" % (nosecmd(), filename, testname)
    print execute(cmd)

def test_module():
    filename, tabstop, line_num = sys.argv[-3:]
    cmd = "%s %s" % (nosecmd(), filename)
    print execute(cmd)

def execute(cmd):
    stdout, stdin = popen2.popen4(cmd)
    test_output = stdout.readlines()
    return write_report(test_output)

